@@ -1,5 +1,5 @@
 package Mooish::AttributeBuilder;
-$Mooish::AttributeBuilder::VERSION = '0.002';
+$Mooish::AttributeBuilder::VERSION = '1.000';
 use v5.10;
 use strict;
 use warnings;
@@ -67,7 +67,15 @@ sub extended
 {
 	my ($name, %args) = @_;
 
-	return ("+$name", expand_shortcuts($name, %args));
+	my $extended_name;
+	if (ref $name eq 'ARRAY') {
+		$extended_name = [map { "+$_" } @{$name}];
+	}
+	else {
+		$extended_name = "+$name";
+	}
+
+	return ($extended_name, expand_shortcuts($name, %args));
 }
 
 # Helpers - not part of the interface
@@ -76,43 +84,30 @@ sub check_and_replace
 {
 	my ($hash_ref, $name, $key, $value) = @_;
 
-	croak "$key already exists for $name"
+	croak "Could not expand shortcut: $key already exists for $name"
 		if exists $hash_ref->{$key};
 
 	$hash_ref->{$key} = $value;
 }
 
-sub expand_shortcuts
+sub get_normalized_name
+{
+	my ($name, $for) = @_;
+
+	croak "Could not use attribute shortcut with array fields: $for is not supported"
+		if ref $name;
+
+	$name =~ s/^_//;
+	return $name;
+}
+
+sub expand_method_names
 {
 	my ($name, %args) = @_;
-	my $normalized_name = $name;
-	$normalized_name =~ s/^_//;
 
-	my $protected_field = $name ne $normalized_name;
-
-	# merge lazy + default / lazy + builder
-	if ($args{lazy}) {
-		my $lazy = $args{lazy};
-		$args{lazy} = 1;
-
-		if (ref $lazy eq 'CODE') {
-			check_and_replace \%args, $name, default => $lazy;
-		}
-		else {
-			check_and_replace \%args, $name, builder => $lazy;
-		}
-	}
-
-	# merge coerce + isa
-	if (blessed $args{coerce}) {
-		check_and_replace \%args, $name, isa => $args{coerce};
-		$args{coerce} = 1;
-	}
-
-	# make sure params with defaults are not required
-	if ($args{required} && (exists $args{default} || $args{builder})) {
-		delete $args{required};
-	}
+	# initialized lazily
+	my $normalized_name;
+	my $protected_field;
 
 	# inflate names from shortcuts
 	for my $method_type (keys %METHOD_PREFIXES) {
@@ -120,6 +115,9 @@ sub expand_shortcuts
 		next if ref $args{$method_type};
 		next unless grep { $_ eq $args{$method_type} } '1', -public, -hidden;
 
+		$normalized_name //= get_normalized_name($name, $method_type);
+		$protected_field //= $name ne $normalized_name;
+
 		my $is_protected =
 			$args{$method_type} eq -hidden
 			|| (
@@ -144,6 +142,47 @@ sub expand_shortcuts
 	return %args;
 }
 
+sub expand_shortcuts
+{
+	my ($name, %args) = @_;
+
+	# merge lazy + default / lazy + builder
+	if ($args{lazy}) {
+		my $lazy = $args{lazy};
+		$args{lazy} = 1;
+
+		if (ref $lazy eq 'CODE') {
+			check_and_replace \%args, $name, default => $lazy;
+		}
+		else {
+			check_and_replace \%args, $name, builder => $lazy;
+		}
+	}
+
+	# merge coerce + isa
+	if (blessed $args{coerce}) {
+		check_and_replace \%args, $name, isa => $args{coerce};
+		$args{coerce} = 1;
+	}
+
+	# make sure params with defaults are not required
+	if ($args{required} && (exists $args{default} || $args{builder})) {
+		delete $args{required};
+	}
+
+	# method names from shortcuts
+	%args = expand_method_names($name, %args);
+
+	# literal parameters (prepended with -)
+	for my $literal (keys %args) {
+		if ($literal =~ m{\A - (.+) \z}x) {
+			$args{$1} = delete $args{$literal};
+		}
+	}
+
+	return %args;
+}
+
 1;
 
 # ABSTRACT: build Mooish attribute definitions with less boilerplate
